@@ -1,26 +1,6 @@
-/*
-    Copyright (c) 2007-2014 Contributors as noted in the AUTHORS file
-
-    This file is part of 0MQ.
-
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
-
 package org.zeromq;
 
 import java.nio.channels.SelectableChannel;
-import java.nio.channels.Selector;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Iterator;
@@ -29,73 +9,76 @@ import java.util.List;
 import org.zeromq.ZMQ.Socket;
 import org.zeromq.ZPoller.EventsHandler;
 
+import zmq.util.Objects;
+
 /**
- * First implementation of a background actor remotely controlled for 0MQ.
- * <p>
- * This implementation is based on the {@link ZStar} one (TODO via inheritance now but this is not totally stamped and the use of the ZAgent would probably limit the collateral damages if changed)
- * so you should be first familiar with the metaphor there.
+ * <p>First implementation of a background actor remotely controlled for ØMQ.</p>
+ *
+ * <p>This implementation is based on the {@link ZStar} one (TODO via inheritance now but this is not totally stamped and the use of the ZAgent would probably limit the collateral damages if changed)
+ * so you should be first familiar with the metaphor there.</p>
+ *
+ * <p>To extensively sum up:
  * <br>
- * To extensively sum up:
- * <p>
  * A side or endpoint designates the same thing: the thread where lives one of the two parts of the Actor system.
- * <br/>
+ * </p>
  * An actor has 2 sides (with a trial to use theater terms for fun (: and hopefully clarity :)
- * <br/>
- * - the Corbeille side, or control side
- * <br/>
+ * <ul>
+ * <li>the Corbeille side, or control side
+ * <br>
  * This is where one can {@link #send(ZMsg) send} and {@link #recv() receive} control messages to the underneath actor via the ZActor and/or its agent.
- * <br/>
+ * <br>
  * The ZActor lives on this side and is a way to safely communicate with the distant provided Actor
- * <br/>
+ * <br>
  * Note: Corbeille is a french word designing the most privileged seats in the theater,
  * 1st floor, just above the orchestra (Wikipedia...).
  * It was the place where the King was standing during the shows, with the best seat of course!
- *
+ * <p>
  * Fast users who would want to communicate with the distant {@link Actor} can use
- * {@link #send(ZMsg)} or {@link #recv()} from the ZActor as the ZActor is itself an agent!
- * <p>
- * - the Plateau side, or background side where all the performances are made by the provided actor.
- * <br/>
- * The provided {@link Actor} is living on the Plateau.
- * <br/>
- * The Plateau is made of the Stage where {@link Actor#stage(Socket, Socket, ZPoller, int) the effective processing occurs} and of the Backstage
- * where the provided actor can {@link Actor#backstage(Socket, ZPoller, int) send and receive} control messages to and from the ZActor.
- * <br/>
- * From this side, the work is done via callbacks using the template-method pattern, applied with interfaces (?)
- * <p>
+ * {@link #send(ZMsg)} or {@link #recv()} from the ZActor as the ZActor is itself an agent!</li>
  *
- * The main purpose (or at least its intent) of this class is to clarify the contracts, roles, responsibilities and action levers related to the Plateau.
- * <p>
+ * <li>the Plateau side, or background side where all the performances are made by the provided actor.
+ * <br>
+ * The provided {@link Actor} is living on the Plateau.
+ * <br>
+ * The Plateau is made of the Stage where {@link org.zeromq.ZActor.Actor#stage(org.zeromq.ZMQ.Socket, org.zeromq.ZMQ.Socket, org.zeromq.ZPoller, int) the effective processing occurs} and of the Backstage
+ * where the provided actor can {@link org.zeromq.ZActor.Actor#backstage(org.zeromq.ZMQ.Socket, org.zeromq.ZPoller, int) send and receive} control messages to and from the ZActor.
+ * <br>
+ * From this side, the work is done via callbacks using the template-method pattern, applied with interfaces (?)
+ * </li>
+ * </ul>
+ * <p>The main purpose (or at least its intent) of this class is to clarify the contracts, roles, responsibilities and action levers related to the Plateau.
+ * <br>
  * The provided Actor will not be alone to do the processing of each loop.
- * <br/>
+ * <br>
  * It will be helped by a double responsible for passing the appropriate structures at the right moment.
  * As a developer point of view, this double helps also to limit the complexity of the process.
  * However Double is an uncommon one, capturing all the lights while letting the Actor taking care of all the invisible work in the shadows.
  * And this is one of the points where we begin to reach the limits of the metaphor...
- * <p>
+ * <br>
  * The ZActor takes care of the establishment of the background processing, calling the provided Actor
  * at appropriate times via its Double. It can also manage the {@link #sign() exited} state on the control side,
  * if using the provided {@link #send(ZMsg)} or {@link #recv()} methods.
- * <br/>
+ * <br>
  * It also takes care of the automatic closing of sockets and context if it had to create one.
- * <p>
- * An {@link Actor actor} is basically a contract interface that anyone who uses this ZActor SHALL comply to.<br/>
- * TODO This interface is still a little bit tough, as instead of the 5+2 Star callbacks, here are 10!
+ * <br>
+ * An {@link Actor actor} is basically a contract interface that anyone who uses this ZActor SHALL comply to.<br>
+ * </p>
+ *
+ * <p>TODO This interface is still a little bit tough, as instead of the 5+2 Star callbacks, here are 10!
  * But they allow you to have a hand on the key points of the looping, restart a new actor if needed, ...
  * Anyway, adapters like a {@link SimpleActor simple one} or a {@link Duo duo} are present to help you on that, reducing the amount of
- * methods to write.
- * <p>
- * PS: Je sais qu'il y a une différence entre acteur et comédien :)
- * PPS: I know nothing about theater!
- * <p>
- * Example of code for a minimalistic actor with no socket handling other than the control pipe:
+ * methods to write.</p>
+ *
+ * <p>PS: Je sais qu'il y a une différence entre acteur et comédien :)</p>
+ * <p>PPS: I know nothing about theater!</p>
+ *
+ * <p>Example of code for a minimalistic actor with no socket handling other than the control pipe:</p>
  *
- * <p>
  * <pre>
  * {@code
         Actor acting = new ZActor.SimpleActor()
         {
-            public List<Socket> createSockets(ZContext ctx, Object[] args)
+            public List<Socket> createSockets(ZContext ctx, Object ... args)
             {
                 assert ("TEST".equals(args[0]));
                 return Arrays.asList(ctx.createSocket(ZMQ.PUB));
@@ -126,7 +109,7 @@ import org.zeromq.ZPoller.EventsHandler;
         rc = actor.send("whatever");
         assert (!rc);
         // don't try to use the pipe
-}
+}</pre>
  */
 // remote controlled background message processing API for 0MQ.
 public class ZActor extends ZStar
@@ -143,10 +126,12 @@ public class ZActor extends ZStar
         /**
          * This is the grand premiere!
          * Called before the creation of the first double and the sockets
-         * 2nd in the order call of the global acting.
+         * 2nd in the order call of the global acting. It return the name
+         * that will be used for the thread. If it's null, a default one
+         * will be picked by ZActor.
          *
          * @param pipe   the backstage control pipe
-         * @return the name of the upcoming performance
+         * @return the name of the upcoming performance.
          */
         String premiere(Socket pipe);
 
@@ -160,7 +145,7 @@ public class ZActor extends ZStar
          * @param args   the arguments passed as parameters of the ZActor
          * @return a list of created sockets that will be managed by the double. Not null.
          */
-        List<Socket> createSockets(ZContext ctx, Object[] args);
+        List<Socket> createSockets(ZContext ctx, Object... args);
 
         /**
          * Called when the double is started, before the first loop.
@@ -181,7 +166,7 @@ public class ZActor extends ZStar
          *
          * @param pipe   the backstage control pipe
          * @param poller the poller of the double
-         * @return the timeout of the coming loop. <b>-1 to block, 0 to not wait, > 0 to wait</b> till max the returned duration in milliseconds
+         * @return the timeout of the coming loop. <b>-1 to block, 0 to not wait, &gt; 0 to wait</b> till max the returned duration in milliseconds
          */
         long looping(Socket pipe, ZPoller poller);
 
@@ -192,7 +177,7 @@ public class ZActor extends ZStar
          * @param pipe    the backstage control pipe receiving the message
          * @param poller  the poller of the double.
          * @param events  the events source of the call
-         * @return true in case of success, <b>false to stop the double</b>.
+         * @return true in case of success, <b>false to stop the actor</b>.
          */
         boolean backstage(Socket pipe, ZPoller poller, int events);
 
@@ -204,7 +189,7 @@ public class ZActor extends ZStar
          * @param pipe    the backstage control pipe
          * @param poller  the poller of the double.
          * @param events  the events source of the call
-         * @return true in case of success, <b>false to stop the double</b>.
+         * @return true in case of success, <b>false to stop the actor</b>.
          */
         boolean stage(Socket socket, Socket pipe, ZPoller poller, int events);
 
@@ -214,7 +199,7 @@ public class ZActor extends ZStar
          *
          * @param pipe   the backstage control pipe
          * @param poller the poller of the double.
-         * @return true to continue with the current double, <b>false to stop it</b>.
+         * @return true to continue with the current doppelganger, <b>false to stop it</b>.
          */
         boolean looped(Socket pipe, ZPoller poller);
 
@@ -230,11 +215,12 @@ public class ZActor extends ZStar
          * Called when the current double has been destroyed.
          * Last in the order call of the double.
          *
-         * @param pipe   the backstage control pipe
+         * @param ctx    the context.
+         * @param pipe   the backstage control pipe.
          * @param poller the poller of the double.
          * @return <b>true to restart a new double</b>, false to stop the acting process
          */
-        boolean destroyed(Socket pipe, ZPoller poller);
+        boolean destroyed(ZContext ctx, Socket pipe, ZPoller poller);
 
         /**
          * Called when the stage is finished.
@@ -256,20 +242,17 @@ public class ZActor extends ZStar
         @Override
         public String premiere(final Socket pipe)
         {
-            // do nothing
-            return "?";
+            return null;
         }
 
         @Override
-        public List<Socket> createSockets(final ZContext ctx,
-                final Object[] args)
+        public List<Socket> createSockets(final ZContext ctx, final Object... args)
         {
             return Collections.emptyList();
         }
 
         @Override
-        public void start(final Socket pipe, final List<Socket> sockets,
-                final ZPoller poller)
+        public void start(final Socket pipe, final List<Socket> sockets, final ZPoller poller)
         {
             // do nothing
         }
@@ -282,16 +265,14 @@ public class ZActor extends ZStar
         }
 
         @Override
-        public boolean backstage(final Socket pipe, final ZPoller poller,
-                final int events)
+        public boolean backstage(final Socket pipe, final ZPoller poller, final int events)
         {
             // stop looping
             return false;
         }
 
         @Override
-        public boolean stage(final Socket socket, final Socket pipe,
-                final ZPoller poller, int events)
+        public boolean stage(final Socket socket, final Socket pipe, final ZPoller poller, int events)
         {
             // stop looping
             return false;
@@ -311,7 +292,7 @@ public class ZActor extends ZStar
         }
 
         @Override
-        public boolean destroyed(final Socket pipe, ZPoller poller)
+        public boolean destroyed(final ZContext ctx, final Socket pipe, final ZPoller poller)
         {
             // no restart
             return false;
@@ -328,6 +309,7 @@ public class ZActor extends ZStar
     /**
      * Another actor will be called just before the main one,
      * without participating to the decisions.
+     * This is interesting as a shadowed observer of the actor's behavior.
      */
     // contract implementation for a duo actor on the stage
     public static class Duo implements Actor
@@ -340,8 +322,8 @@ public class ZActor extends ZStar
         public Duo(final Actor main, final Actor shadow)
         {
             super();
-            assert (main != null);
-            assert (shadow != null);
+            Objects.requireNonNull(main, "Actor shall be set to a non-null value");
+            Objects.requireNonNull(shadow, "Shadow Actor shall be set to a non-null value");
             this.main = main;
             this.shadow = shadow;
         }
@@ -354,16 +336,14 @@ public class ZActor extends ZStar
         }
 
         @Override
-        public List<Socket> createSockets(final ZContext ctx,
-                final Object[] args)
+        public List<Socket> createSockets(final ZContext ctx, final Object... args)
         {
             shadow.createSockets(ctx, args);
             return main.createSockets(ctx, args);
         }
 
         @Override
-        public void start(final Socket pipe, final List<Socket> sockets,
-                final ZPoller poller)
+        public void start(final Socket pipe, final List<Socket> sockets, final ZPoller poller)
         {
             shadow.start(pipe, sockets, poller);
             main.start(pipe, sockets, poller);
@@ -377,16 +357,14 @@ public class ZActor extends ZStar
         }
 
         @Override
-        public boolean backstage(final Socket pipe, final ZPoller poller,
-                final int events)
+        public boolean backstage(final Socket pipe, final ZPoller poller, final int events)
         {
             shadow.backstage(pipe, poller, events);
             return main.backstage(pipe, poller, events);
         }
 
         @Override
-        public boolean stage(final Socket socket, final Socket pipe,
-                final ZPoller poller, final int events)
+        public boolean stage(final Socket socket, final Socket pipe, final ZPoller poller, final int events)
         {
             shadow.stage(socket, pipe, poller, events);
             return main.stage(socket, pipe, poller, events);
@@ -407,10 +385,10 @@ public class ZActor extends ZStar
         }
 
         @Override
-        public boolean destroyed(final Socket pipe, final ZPoller poller)
+        public boolean destroyed(final ZContext ctx, final Socket pipe, final ZPoller poller)
         {
-            shadow.destroyed(pipe, poller);
-            return main.destroyed(pipe, poller);
+            shadow.destroyed(ctx, pipe, poller);
+            return main.destroyed(ctx, pipe, poller);
         }
 
         @Override
@@ -422,16 +400,18 @@ public class ZActor extends ZStar
     }
 
     /**
-     * Creates a new ZActor.
+     * Creates a new ZActor. A new context will be created and closed at the stop of the operation.
      *
      * @param actor
      *            the actor handling messages from either stage and backstage
+     * @param motdelafin
+     *            the final word used to mark the end of the actor. Null to disable this mechanism.
      * @param args
      *            the optional arguments that will be passed to the distant actor
      */
-    public ZActor(Actor actor, final String motdelafin, Object... args)
+    public ZActor(final Actor actor, final String motdelafin, final Object... args)
     {
-        this(null, null, actor, motdelafin, args);
+        super(new ActorFortune(actor), motdelafin, args);
     }
 
     /**
@@ -441,12 +421,16 @@ public class ZActor extends ZStar
      *            the creator of the selector used on the Plateau.
      * @param actor
      *            the actor handling messages from either stage and backstage
+     * @param motdelafin
+     *            the final word used to mark the end of the actor. Null to disable this mechanism.
      * @param args
      *            the optional arguments that will be passed to the distant actor
+     * @deprecated use {@link ZActor#ZActor(Actor, String, Object...)}
      */
+    @Deprecated
     public ZActor(final SelectorCreator selector, final Actor actor, final String motdelafin, final Object... args)
     {
-        this(null, selector, actor, motdelafin, args);
+        this(actor, motdelafin, args);
     }
 
     /**
@@ -461,13 +445,37 @@ public class ZActor extends ZStar
      *            the creator of the selector used on the Plateau.
      * @param actor
      *            the actor handling messages from either stage and backstage
+     * @param motdelafin
+     *            the final word used to mark the end of the actor. Null to disable this mechanism.
+     * @param args
+     *            the optional arguments that will be passed to the distant actor
+     * @deprecated use {@link ZActor#ZActor(ZContext, Actor, String, Object...)}
+     */
+    @Deprecated
+    public ZActor(final ZContext context, final SelectorCreator selector, final Actor actor, final String motdelafin,
+                  final Object... args)
+    {
+        this(context, actor, motdelafin, args);
+    }
+
+    /**
+     * Creates a new ZActor.
+     *
+     * @param context
+     *            the main context used. If null, a new context will be created
+     *            and closed at the stop of the operation.
+     * <b>If not null, it is the responsibility of the caller to close it.</b>
+     *
+     * @param actor
+     *            the actor handling messages from either stage and backstage
+     * @param motdelafin
+     *            the final word used to mark the end of the actor. Null to disable this mechanism.
      * @param args
      *            the optional arguments that will be passed to the distant actor
      */
-    public ZActor(final ZContext context, final SelectorCreator selector,
-            final Actor actor, final String motdelafin, final Object[] args)
+    public ZActor(final ZContext context, final Actor actor, final String motdelafin, final Object... args)
     {
-        super(context, selector, new ActorFortune(actor), motdelafin, args);
+        super(context, new ActorFortune(actor), motdelafin, args);
     }
 
     // actor creator
@@ -477,22 +485,20 @@ public class ZActor extends ZStar
 
         public ActorFortune(Actor actor)
         {
-            assert (actor != null);
+            Objects.requireNonNull(actor, "Actor shall be set to a non-null value");
             this.actor = actor;
         }
 
         @Override
-        public String premiere(Socket mic, Object[] args)
+        public String premiere(Socket mic, Object... args)
         {
             return actor.premiere(mic);
         }
 
         @Override
-        public Star create(ZContext ctx, Socket pipe, Selector sel,
-                int count, Star previous, Object[] args)
+        public Star create(ZContext ctx, Socket pipe, int count, Star previous, Object... args)
         {
-            Star star = new ZActor.Double(ctx, pipe, sel, actor, args);
-            return star;
+            return new ZActor.Double(ctx, pipe, actor, args);
         }
 
         @Override
@@ -522,20 +528,22 @@ public class ZActor extends ZStar
         // actor responsible for processing messages
         private final Actor actor;
 
+        // context used for the closing of the sockets
+        private final ZContext context;
+
         // creates a new double
-        public Double(final ZContext ctx, final Socket pipe,
-                final Selector selector, final Actor actor,
-                final Object[] args)
+        public Double(final ZContext ctx, final Socket pipe, final Actor actor, final Object... args)
         {
+            this.context = ctx;
             this.pipe = pipe;
             this.actor = actor;
 
             final List<Socket> created = actor.createSockets(ctx, args);
             assert (created != null);
 
-            sockets = new ArrayList<Socket>(created);
+            sockets = new ArrayList<>(created);
 
-            poller = new ZPoller(selector);
+            poller = new ZPoller(ctx);
             poller.setGlobalHandler(this);
         }
 
@@ -588,7 +596,7 @@ public class ZActor extends ZStar
                 }
             }
             // let the actor decide if the stage restarts a new double
-            return actor.destroyed(pipe, poller);
+            return actor.destroyed(context, pipe, poller);
         }
 
         @Override
