@@ -1,22 +1,3 @@
-/*
-    Copyright (c) 2007-2014 Contributors as noted in the AUTHORS file
-
-    This file is part of 0MQ.
-
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
-
 package org.zeromq;
 
 import java.io.DataInputStream;
@@ -28,10 +9,14 @@ import java.util.ArrayDeque;
 import java.util.Collection;
 import java.util.Deque;
 import java.util.Iterator;
-import java.util.NoSuchElementException;
+import java.util.Objects;
+import java.util.stream.Collectors;
 
 import org.zeromq.ZMQ.Socket;
 
+import zmq.util.Draft;
+import zmq.util.function.Consumer;
+
 /**
  * The ZMsg class provides methods to send and receive multipart messages
  * across 0MQ sockets. This class provides a list-like container interface,
@@ -44,7 +29,7 @@ import org.zeromq.ZMQ.Socket;
  *
  * // Add several frames into one message
  * ZMsg msg = new ZMsg();
- * for (int i = 0;i< 10;i++) {
+ * for (int i = 0 ; i &lt; 10 ; i++) {
  *     msg.addString("Frame" + i);
  * }
  * msg.send(output);
@@ -65,14 +50,13 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     /**
      * Hold internal list of ZFrame objects
      */
-    private ArrayDeque<ZFrame> frames;
+    private final ArrayDeque<ZFrame> frames = new ArrayDeque<>();
 
     /**
      * Class Constructor
      */
     public ZMsg()
     {
-        frames = new ArrayDeque<ZFrame>();
     }
 
     /**
@@ -81,14 +65,10 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
      */
     public void destroy()
     {
-        if (frames == null) {
-            return;
-        }
         for (ZFrame f : frames) {
             f.destroy();
         }
         frames.clear();
-        frames = null;
     }
 
     /**
@@ -108,12 +88,10 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
      * @param str
      *              String to add to list
      */
-    public void addString(String str)
+    public ZMsg addString(String str)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         frames.add(new ZFrame(str));
+        return this;
     }
 
     /**
@@ -124,16 +102,16 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
      */
     public ZMsg duplicate()
     {
-        if (frames != null) {
+        if (frames.isEmpty()) {
+            return null;
+        }
+        else {
             ZMsg msg = new ZMsg();
             for (ZFrame f : frames) {
                 msg.add(f.duplicate());
             }
             return msg;
         }
-        else {
-            return null;
-        }
     }
 
     /**
@@ -141,12 +119,13 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
      * Message takes ownership of frame, will destroy it when message is sent.
      * @param frame
      */
-    public void wrap(ZFrame frame)
+    public ZMsg wrap(ZFrame frame)
     {
         if (frame != null) {
             push(new ZFrame(""));
             push(frame);
         }
+        return this;
     }
 
     /**
@@ -195,11 +174,7 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
             throw new IllegalArgumentException("socket is null");
         }
 
-        if (frames == null) {
-            throw new IllegalArgumentException("destroyed message");
-        }
-
-        if (frames.size() == 0) {
+        if (frames.isEmpty()) {
             return true;
         }
 
@@ -208,6 +183,9 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
         while (i.hasNext()) {
             ZFrame f = i.next();
             ret = f.sendAndKeep(socket, (i.hasNext()) ? ZFrame.MORE : 0);
+            if (!ret) {
+                break;
+            }
         }
         if (destroy) {
             destroy();
@@ -228,6 +206,19 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
         return recvMsg(socket, 0);
     }
 
+    /**
+     * Receives message from socket, returns ZMsg object or null if the
+     * recv was interrupted.
+     * @param   socket
+     * @param   wait true to wait for next message, false to do a non-blocking recv.
+     * @return
+     *          ZMsg object, null if interrupted
+     */
+    public static ZMsg recvMsg(Socket socket, boolean wait)
+    {
+        return recvMsg(socket, wait ? 0 : ZMQ.DONTWAIT);
+    }
+
     /**
      * Receives message from socket, returns ZMsg object or null if the
      * recv was interrupted. Setting the flag to ZMQ.DONTWAIT does a non-blocking recv.
@@ -261,8 +252,44 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     }
 
     /**
-     * Save message to an open data output stream.
+     * This API is in DRAFT state and is subject to change at ANY time until declared stable
+     * handle incoming message with a handler
      *
+     * @param socket
+     * @param flags see ZMQ constants
+     * @param handler handler to handle incoming message
+     * @param exceptionHandler handler to handle exceptions
+     */
+    @Draft
+    public static void recvMsg(ZMQ.Socket socket, int flags,
+                               Consumer<ZMsg> handler,
+                               Consumer<ZMQException> exceptionHandler)
+    {
+        try {
+            handler.accept(ZMsg.recvMsg(socket, flags));
+        }
+        catch (ZMQException e) {
+            exceptionHandler.accept(e);
+        }
+    }
+
+    /**
+     * This API is in DRAFT state and is subject to change at ANY time until declared stable
+     * handle incoming message with a handler
+     *
+     * @param socket
+     * @param flags see ZMQ constants
+     * @param handler handler to handle incoming message
+     */
+    @Draft
+    public static void recvMsg(ZMQ.Socket socket, int flags, Consumer<ZMsg> handler)
+    {
+        handler.accept(ZMsg.recvMsg(socket, flags));
+    }
+
+    /**
+     * Save message to an open data output stream.
+     * <p>
      * Data saved as:
      *      4 bytes: number of frames
      *  For every frame:
@@ -361,17 +388,13 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
         }
         ZMsg zMsg = (ZMsg) o;
 
-        if (frames == null || zMsg.frames == null) {
-            return false;
-        }
-
         //based on AbstractList
         Iterator<ZFrame> e1 = frames.iterator();
         Iterator<ZFrame> e2 = zMsg.frames.iterator();
         while (e1.hasNext() && e2.hasNext()) {
             ZFrame o1 = e1.next();
             ZFrame o2 = e2.next();
-            if (!(o1 == null ? o2 == null : o1.equals(o2))) {
+            if (!(Objects.equals(o1, o2))) {
                 return false;
             }
         }
@@ -381,7 +404,7 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public int hashCode()
     {
-        if (frames == null || frames.size() == 0) {
+        if (frames.isEmpty()) {
             return 0;
         }
 
@@ -397,62 +420,68 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
      * Dump the message in human readable format. This should only be used
      * for debugging and tracing, inefficient in handling large messages.
      **/
-    public void dump(Appendable out)
+    public ZMsg dump(Appendable out)
     {
         try {
             StringWriter sw = new StringWriter();
             PrintWriter pw = new PrintWriter(sw);
             pw.printf("--------------------------------------\n");
             for (ZFrame frame : frames) {
-                pw.printf("[%03d] %s\n", frame.size(),
-                        frame.toString());
+                pw.printf("[%03d] %s\n", frame.size(), frame);
             }
             out.append(sw.getBuffer());
             sw.close();
         }
         catch (IOException e) {
-            throw new RuntimeException("Message dump exception "
-                    + super.toString(), e);
+            throw new RuntimeException("Message dump exception " + super.toString(), e);
         }
+        return this;
     }
 
-    public void dump()
+    public ZMsg dump()
     {
         dump(System.out);
+        return this;
     }
 
     // ********* Convenience Deque methods for common data types *** //
 
-    public void addFirst(String stringValue)
+    public ZMsg addFirst(String stringValue)
     {
         addFirst(new ZFrame(stringValue));
+        return this;
     }
 
-    public void addFirst(byte[] data)
+    public ZMsg addFirst(byte[] data)
     {
         addFirst(new ZFrame(data));
+        return this;
     }
 
-    public void addLast(String stringValue)
+    public ZMsg addLast(String stringValue)
     {
         addLast(new ZFrame(stringValue));
+        return this;
     }
 
-    public void addLast(byte[] data)
+    public ZMsg addLast(byte[] data)
     {
         addLast(new ZFrame(data));
+        return this;
     }
 
     // ********* Convenience Queue methods for common data types *** //
 
-    public void push(String str)
+    public ZMsg push(String str)
     {
         push(new ZFrame(str));
+        return this;
     }
 
-    public void push(byte[] data)
+    public ZMsg push(byte[] data)
     {
         push(new ZFrame(data));
+        return this;
     }
 
     public boolean add(String stringValue)
@@ -465,11 +494,34 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
         return add(new ZFrame(data));
     }
 
+    /**
+     * Adds a string as a new frame in the message.
+     *
+     * @param stringValue the value to add
+     * @return this
+     */
+    public ZMsg append(String stringValue)
+    {
+        add(stringValue);
+        return this;
+    }
+
+    /**
+     * Adds bytes as a new frame in the message.
+     *
+     * @param data the value to add
+     * @return this
+     */
+    public ZMsg append(byte[] data)
+    {
+        add(data);
+        return this;
+    }
+
     // ********* Implement Iterable Interface *************** //
     @Override
     public Iterator<ZFrame> iterator()
     {
-        // TODO Auto-generated method stub
         return frames.iterator();
     }
 
@@ -525,27 +577,18 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public boolean add(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         return frames.add(e);
     }
 
     @Override
     public void addFirst(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         frames.addFirst(e);
     }
 
     @Override
     public void addLast(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         frames.addLast(e);
     }
 
@@ -570,49 +613,30 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public ZFrame getFirst()
     {
-        try {
-            return frames.getFirst();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.peekFirst();
     }
 
     @Override
     public ZFrame getLast()
     {
-        try {
-            return frames.getLast();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.peekLast();
     }
 
     @Override
     public boolean offer(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         return frames.offer(e);
     }
 
     @Override
     public boolean offerFirst(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         return frames.offerFirst(e);
     }
 
     @Override
     public boolean offerLast(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         return frames.offerLast(e);
     }
 
@@ -625,23 +649,13 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public ZFrame peekFirst()
     {
-        try {
-            return frames.peekFirst();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.peekFirst();
     }
 
     @Override
     public ZFrame peekLast()
     {
-        try {
-            return frames.peekLast();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.peekLast();
     }
 
     @Override
@@ -665,15 +679,7 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public ZFrame pop()
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
-        try {
-            return frames.pop();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.poll();
     }
 
     /**
@@ -694,9 +700,6 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public void push(ZFrame e)
     {
-        if (frames == null) {
-            frames = new ArrayDeque<ZFrame>();
-        }
         frames.push(e);
     }
 
@@ -715,12 +718,7 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public ZFrame removeFirst()
     {
-        try {
-            return frames.removeFirst();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.pollFirst();
     }
 
     @Override
@@ -732,12 +730,7 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public ZFrame removeLast()
     {
-        try {
-            return frames.removeLast();
-        }
-        catch (NoSuchElementException e) {
-            return null;
-        }
+        return frames.pollLast();
     }
 
     @Override
@@ -752,6 +745,21 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
         return frames.size();
     }
 
+    public ZMsg append(ZMsg msg)
+    {
+        if (msg == null) {
+            return this;
+        }
+        // Tests explicitly check appending a ZMsg to itself, protect that
+        if (msg != this) {
+            frames.addAll(msg.frames);
+        }
+        else {
+            frames.addAll(msg.frames.clone());
+        }
+        return this;
+    }
+
     /**
      * Returns pretty string representation of multipart message:
      * [ frame0, frame1, ..., frameN ]
@@ -761,15 +769,7 @@ public class ZMsg implements Iterable<ZFrame>, Deque<ZFrame>
     @Override
     public String toString()
     {
-        StringBuilder out = new StringBuilder("[ ");
-        Iterator<ZFrame> frameIterator = frames.iterator();
-        while (frameIterator.hasNext()) {
-            out.append(frameIterator.next());
-            if (frameIterator.hasNext()) {
-                out.append(", "); // skip last iteration
-            }
-        }
-        out.append(" ]");
-        return out.toString();
+        String joined = frames.stream().map(ZFrame::toString).collect(Collectors.joining(", "));
+        return new StringBuilder("[ ").append(joined).append(" ]").toString();
     }
 }
